﻿#region Using directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using System.Threading;

#endregion

namespace BackgroundThread
{
    partial class FibonacciForm : System.Windows.Forms.Form
    {	
        private int numberToCompute = 0;
        private int highestPercentageReached = 0;

        private System.Windows.Forms.NumericUpDown numericUpDown1;
        private System.Windows.Forms.Button startButton;
        private System.Windows.Forms.Button cancelButton;
        private System.Windows.Forms.ProgressBar progressBar1;
        private System.Windows.Forms.Label result;
        private System.ComponentModel.BackgroundWorker backgroundWorker1;

        public FibonacciForm()
        {	
            InitializeComponent();
            InitializeBackgoundWorker();
        }

        // Eventhandler für backgroundWorker1 initialisieren 
        private void InitializeBackgoundWorker()
        {
            backgroundWorker1.DoWork += new DoWorkEventHandler(backgroundWorker1_DoWork);
            backgroundWorker1.RunWorkerCompleted += new RunWorkerCompletedEventHandler(backgroundWorker1_RunWorkerCompleted);
            backgroundWorker1.ProgressChanged += new ProgressChangedEventHandler(backgroundWorker1_ProgressChanged);
        }
	
        private void startButton_Click(System.Object sender, System.EventArgs e)
        {
            result.Text = String.Empty;
            // Ausschalten, wenn gearbeitet wird.
            this.numericUpDown1.Enabled = false;
            this.startButton.Enabled = false;
            // Einschalten, zum abbrechen.
            this.cancelButton.Enabled = true;

            numberToCompute = (int)numericUpDown1.Value;
            highestPercentageReached = 0;

            // Start der asynchronen Operation.
            backgroundWorker1.RunWorkerAsync(numberToCompute);
        }

        private void cancelButton_Click(System.Object sender, 
            System.EventArgs e)
        {   
            this.backgroundWorker1.CancelAsync();
            cancelButton.Enabled = false;
        }

        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {   
            BackgroundWorker worker = sender as BackgroundWorker;

            // Hier wird gearbeitet...
            // Ergebnis in e.result speichern
            e.Result = ComputeFibonacci((int)e.Argument, worker, e);
        }

        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show(e.Error.Message);
            }
            else if (e.Cancelled)
            {
                result.Text = "Abgebrochen";
            }
            else
            {
                // Alles erfolgreich erledigt
                result.Text = e.Result.ToString();
            }

            this.numericUpDown1.Enabled = true;
            startButton.Enabled = true;
            cancelButton.Enabled = false;
        }

        private void backgroundWorker1_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            this.progressBar1.Value = e.ProgressPercentage;
        }

        long ComputeFibonacci(int n, BackgroundWorker worker, DoWorkEventArgs e)
        {
            // Parameter-Test.
            if ((n < 0) || (n > 91))
            {
                throw new ArgumentException("Wert muss >= 0 und <= 91 sein.", "n");
            }

            long result = 0;

            // Abbrechen-Button geklickt.
            if (worker.CancellationPending)
            {   
                e.Cancel = true;
            }
            else
            {   
                if (n < 2)
                {   
                    result = 1;
                }
                else
                {   
                    result = ComputeFibonacci(n - 1, worker, e) + 
                             ComputeFibonacci(n - 2, worker, e);
                }

                // Progressbar setzen
                int percentComplete = (int)((float)n / (float)numberToCompute * 100);
                
                if (percentComplete > highestPercentageReached)
                {
                    highestPercentageReached = percentComplete;
                    worker.ReportProgress(percentComplete);
                }
            }

            return result;
        }

		#region Windows Form Designer generated code
		
        private void InitializeComponent()
        {
          this.numericUpDown1 = new System.Windows.Forms.NumericUpDown();
          this.startButton = new System.Windows.Forms.Button();
          this.cancelButton = new System.Windows.Forms.Button();
          this.result = new System.Windows.Forms.Label();
          this.progressBar1 = new System.Windows.Forms.ProgressBar();
          this.backgroundWorker1 = new System.ComponentModel.BackgroundWorker();
          ((System.ComponentModel.ISupportInitialize)(this.numericUpDown1)).BeginInit();
          this.SuspendLayout();
// 
// numericUpDown1
// 
          this.numericUpDown1.Location = new System.Drawing.Point(18, 16);
          this.numericUpDown1.Maximum = new decimal(new int[] {
            91,
            0,
            0,
            0});
          this.numericUpDown1.Minimum = new decimal(new int[] {
            1,
            0,
            0,
            0});
          this.numericUpDown1.Name = "numericUpDown1";
          this.numericUpDown1.Size = new System.Drawing.Size(80, 20);
          this.numericUpDown1.TabIndex = 0;
          this.numericUpDown1.Value = new decimal(new int[] {
            1,
            0,
            0,
            0});
// 
// startButton
// 
          this.startButton.Location = new System.Drawing.Point(18, 72);
          this.startButton.Name = "startButton";
          this.startButton.Size = new System.Drawing.Size(120, 23);
          this.startButton.TabIndex = 1;
          this.startButton.Text = "Start Asynchron";
          this.startButton.Click += new System.EventHandler(this.startButton_Click);
// 
// cancelButton
// 
          this.cancelButton.Enabled = false;
          this.cancelButton.Location = new System.Drawing.Point(153, 72);
          this.cancelButton.Name = "cancelButton";
          this.cancelButton.Size = new System.Drawing.Size(119, 23);
          this.cancelButton.TabIndex = 2;
          this.cancelButton.Text = "Abbrechen";
          this.cancelButton.Click += new System.EventHandler(this.cancelButton_Click);
// 
// result
// 
          this.result.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
          this.result.Location = new System.Drawing.Point(112, 16);
          this.result.Name = "result";
          this.result.Size = new System.Drawing.Size(160, 23);
          this.result.TabIndex = 3;
          this.result.Text = "(kein Ergebnis)";
          this.result.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
// 
// progressBar1
// 
          this.progressBar1.Location = new System.Drawing.Point(18, 48);
          this.progressBar1.Name = "progressBar1";
          this.progressBar1.Size = new System.Drawing.Size(254, 17);
          this.progressBar1.Step = 2;
          this.progressBar1.TabIndex = 4;
// 
// backgroundWorker1
// 
          this.backgroundWorker1.WorkerReportsProgress = true;
          this.backgroundWorker1.WorkerSupportsCancellation = true;
// 
// FibonacciForm
// 
          this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
          this.ClientSize = new System.Drawing.Size(289, 106);
          this.Controls.Add(this.progressBar1);
          this.Controls.Add(this.result);
          this.Controls.Add(this.cancelButton);
          this.Controls.Add(this.startButton);
          this.Controls.Add(this.numericUpDown1);
          this.Name = "FibonacciForm";
          this.Text = "Fibonacci Calculator";
          ((System.ComponentModel.ISupportInitialize)(this.numericUpDown1)).EndInit();
          this.ResumeLayout(false);

        }
		#endregion
    }
}